/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;
import java.util.*;

/**
 * A class that produces a page control component.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class PageControl extends Panel {

	/** When user change selected tab in TabControl, 
	 *  parent will get the event with this id. 
	 */
    public static final int TAB_CHANGE = 47421;

	/** The basic color of the 3D border. Left and top sides are 
	 *  lighter and right and bottom sides are darker then this 
	 *  color. By default it is set to defaultBorderColor.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #defaultBorderColor
	 */
	protected Color marginColor = null;

	/** Default color of the border. Currently lightGray.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #marginColor
	 */
    protected static final Color defaultBorderColor = new Color(192, 192, 192);

	/** Current width of the 3D border around the component
	 * @see #defaultBorderWidth
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */	
    protected int margin = 0;

	/** Default value of margin. Currently 1.
	 * @see #margin
	 * @see #setBorderWidth
	 * @see #getBorderWidth
	 */
    protected static final int defaultBorderWidth = 1;

    /** Vector of panels inserted into PageControl. */
    protected Vector pages;

    /** TabControl component for switching among pages. */
    protected TabControl tab;

    private Dimension dimLast = null;

	/** Create new PageControl with all default values. */
    public PageControl() {
        setLayout(null);
        tab = new TabControl();
        add(tab);
        validate();
        pages = new Vector();
        dimLast = new Dimension(0,0);
	    setDefaults();
    }

	/** Select new page.
	 * @param index Index of new selected page.
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 * @see #getTabIndex
	 */
    public void setTabIndex(int index) throws ArrayIndexOutOfBoundsException {
        int old = tab.getTabIndex();
        if (old != index) {
            ((Panel) (pages.elementAt(old))).hide();
    	    tab.setTabIndex(index);
            ((Panel) (pages.elementAt(index))).show();
        }
    }

	/** Gets the index of currently selected page.
	 * @return Index of selected page.
	 * @see #setTabIndex
	 */
    public int getTabIndex() {
	    return tab.getTabIndex();
    }

	/** Gets count of the pages.
	 * @return Count of pages.
	 */
    public int getTabCount() {
        return tab.getTabCount();
    }

	/** Gets the name of the page with the specific index.
	 * @param index Index of page.
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 * @return Name of page.
	 */
    public String getTabName(int index) throws ArrayIndexOutOfBoundsException {
        return tab.getTabName(index);
    }

    private void setDefaults() {
        setBorderWidth(defaultBorderWidth);
        marginColor = defaultBorderColor;
        tab.marginColor = marginColor;
        tab.direction = true;
    }

	/** Add new page.
	 * @param newTab Name of new page.
	 * @param panel Panel which will be inserted into PageControl.
	 */
    public void addPage(String newTab, Panel panel) {
        int index = tab.getTabCount();
        addPageAt(newTab, panel, index);
    }

	/** Add new page at specific position.
	 * @param newTab Name of new page.
	 * @param panel Panel which will be inserted into PageControl.
	 * @param index Position of new page.
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 */
    public void addPageAt(String newTab, Panel panel, int index) throws
             ArrayIndexOutOfBoundsException {
        tab.addTabAt(newTab, index);
        pages.insertElementAt(panel, index);
        add(panel);
        validate();
        countSizes();
        int j = getTabIndex();
        if (j == -1) {
            setTabIndex(index);
            j = index;
        }
        for (int i = 0; i < tab.getTabCount(); i++) {
            if (i != j)
                ((Panel) pages.elementAt(i)).hide();
            else
                ((Panel) pages.elementAt(i)).show();
        }
    }

	/** Gets the panel of specific page.
	 * @param index Position of page.
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 */
    public Panel getPageAt(int index) throws ArrayIndexOutOfBoundsException {
        return (Panel) pages.elementAt(index);
    }

	/** Removes the page at the specific position.
	 * @param index Position of page.
	 * @exception ArrayIndexOutOfBoundsException If the value was invalid.
	 */
    public void removePageAt(int index) throws ArrayIndexOutOfBoundsException {
		boolean b = false;
		if (index == tab.getTabIndex())
			b = true;
		remove((Panel) pages.elementAt(index));
		pages.removeElementAt(index);
		tab.removeTabAt(index);
		if (b) {
			int i = getTabIndex();
			if (i != -1)
				((Panel) (pages.elementAt(i))).show();
		}
		repaint();
    }

	/** Remove all pages.
	 * @see #removePageAt
	 */
	public void removeAllPages() {
		tab.removeAllTabs();
		for (int i = 0; i < pages.size(); i++) 
			remove((Panel) pages.elementAt(i));
		pages.removeAllElements();
		repaint();
	}

	/** Sets border width around the component.
	 * @param borderWidth New value of margin.
	 * @see #margin
	 * @see #getBorderWidth
	 */
    public void setBorderWidth(int borderWidth) {
        margin = borderWidth;
        tab.margin = margin;
    }

	/** Gets border width around the component.
	 * @see #margin
	 * @see #setBorderWidth
	 */
    public int getBorderWidth() {
        return(margin);
    }

	/** Sets the border color.
	 * @param col New value of marginColor.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #getBorderColor
	 */
    public void setBorderColor(Color col) {
        marginColor = col;
        tab.marginColor = col;
    }

	/** Gets the border color.
	 * @return Current border color.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #setBorderColor
	 */
    public Color getBorderColor(Color col) {
        return marginColor;
    }

	/** Sets a new direction of PageControl. If it is true, 
	 *  TabControl if on the top of PageControl otherwise
	 *  at the bottom.
	 * @param direct New direction.
	 * @see #getDirection
	 */
    public void setDirection(boolean direct) {
        if (tab.direction != direct) {
            tab.direction = direct;
            countSizes();
            repaint();
        }
    }

	/** Gets the current direction of PageControl. If it is true, 
	 *  TabControl if on the top of PageControl otherwise
	 *  at the bottom.
	 * @return Current direction.
	 * @see #setDirection
	 */
    public boolean getDirection() {
        return tab.direction;
    }

    private void countSizes() {
        Dimension d = tab.preferredSize();
        Dimension t = size();
        dimLast = t;
        Dimension ts = tab.size();
        if (ts != d)
            tab.resize(t.width, d.height);
        Panel panel;
        int x,y;
        if (tab.direction) {
            if ((tab.location().x != 0) || (tab.location().y != 0))
                tab.move(0,0);
            x = margin;
            y = d.height;
        }
        else {
            if ((tab.location().x != 0) || (tab.location().y != (t.height - d.height)))
                tab.move(0, t.height - d.height);
            x = margin;
            y = margin;
        }
        int w = t.width - 2 * margin;
        int h = t.height - d.height - 2 * margin;
        for (int i = 0; i < getTabCount(); i++) {
            panel = (Panel) pages.elementAt(i);
            panel.reshape(x, y, w, h);
            panel.validate();
        }
    }

    public void paint(Graphics g) {
  		Dimension s = size();
        if (s != dimLast)
            countSizes();
		if (tab.getTabCount() > 0) {
            int left = 0;
            int width = s.width - 1;
            int height = s.height - tab.size().height + margin - 1;
            int top = 0;
            if (tab.direction)
                top = tab.size().height - margin;

            g.setColor(marginColor);
            for(int i = 0; i < margin; i++) {
                g.draw3DRect(left, top, width, height, true);
                left++;
                top++;
                width = width - 2;
                height = height - 2;
            }
        }
        else {
			g.setColor(getBackground());
			g.fillRect(0, 0, s.width, s.height);
		}
    }

    public boolean handleEvent(Event event) {
		if (event.id == TAB_CHANGE) {
			int newIndex = tab.getTabIndex();
			for (int i = 0; i < tab.getTabCount(); i++) {
			if (i == newIndex)
				((Panel) (pages.elementAt(i))).show();
			else
				((Panel) (pages.elementAt(i))).hide();
			}
		}
		return(false);
    }

	public void reshape(int x, int y, int width, int height) {
		super.reshape(x, y, width, height);
		countSizes();
	}

    public void resize(int width, int height) {
		super.resize(width, height);
		countSizes();
    }

    public void resize(Dimension d) {
		super.resize(d);
		countSizes();
    }

    public Dimension minimumSize() {
        return countSize(true);
    }

    public Dimension preferredSize() {
        return countSize(false);
    }

    private Dimension countSize(boolean min_pref) {
        Dimension d = (min_pref ? tab.minimumSize() : tab.preferredSize());
        Dimension e = new Dimension(0, 0);
        for (int i = 0; i < tab.getTabCount(); i++) {
            Panel p = (Panel) pages.elementAt(i);
            Dimension f = (min_pref ? p.minimumSize() : p.preferredSize());
            if (e.width < f.width)
                e.width = f.width;
            if (e.height < f.height)
                e.height = f.height;
        }
        return e;
    }

}
