package xelfi.awt;

import java.awt.*;

import xelfi.util.Debug;

/**
 * A panel for viewing hierarchically sorted strings in tree structure
 * with vertical scrollbar.<br>
 *
 * @version 1.01, 19 Nov 1996
 * @author Dafe
	Last change:  DF   17 Mar 97   12:24 pm
*/
public class XFTreeViewPanel extends Panel {

	private Dimension						oldTreeSize = new Dimension(0, 0);
	private Dimension						oldSize = new Dimension(0, 0);

  private int                 LINE_INCREMENT = 15;
  
	// a canvas that can draw tree structure
	protected XFTreeViewCanvas	treeView;
	// vertical scrollbar
	private Scrollbar						vScrollbar, hScrollbar;
	private boolean							vHidden;
	// enables/disables user input (scrollbar's reactions)
	private boolean							userInput;

	/**
	 * Constructs new tree view panel with specified 
	 * root element.<br>
	 * Creates scrollbar and tree view canvas.
	*/	
	public XFTreeViewPanel (Object rootElement) {
		vScrollbar = new Scrollbar(Scrollbar.VERTICAL); 
		vHidden = false;
		treeView = new XFTreeViewCanvas(rootElement, this);
		setLayout(new BorderLayout());
		add("Center", treeView);
		add("East", vScrollbar);
	}

  /**
   * Empties content and sets new root.
  */
  public void newRoot (Object newRootElement) {
    remove(treeView);
		boolean multiSel = allowsMultipleSelections();
		boolean dragDrop = allowsDragDrop();
		boolean userInput = getUserInput();
    treeView = new XFTreeViewCanvas(newRootElement, this);
		setDragDrop(dragDrop);
		setMultipleSelections(multiSel);
		enableUserInput(userInput);
    add("Center", treeView);
  }

  /**
   * Sets new tree.
  */
  public void setTree (XFTree newTree) {
    treeView.setTree(newTree);
  }

  /**
	 * Enables/disables user input. Set the same value to 
	 * tree view canvas.
	*/
	public void enableUserInput (boolean userInput) {
		treeView.enableUserInput(userInput);
	}

	/**
	 * Returns user input state.
	*/
	public boolean getUserInput () {
		return treeView.getUserInput();
	}

	/**
	 * Returns canvas which serves as tree view
	*/
	public XFTreeViewCanvas getTreeViewCanvas () {
		return treeView;
	}

  public boolean handleEvent (Event e) {
		// propagate only if input is disabled
		if (treeView.getUserInput()==false) return false;

    if (e.target == hScrollbar)
    {
			treeView.setView(hScrollbar.getValue(),
												treeView.getViewY());
    }
    else if(e.target == vScrollbar)
    {
			treeView.setView(treeView.getViewX(),
												vScrollbar.getValue()*LINE_INCREMENT);
    }
		// propagate
		return false;
	}


	/**
	 * Inserts new view element after currently selected
	 * and selects it
	*/
	public void insertAfter (XelfiTreeViewElement element) {
		treeView.insertAfter(element);
	}

	/**
	 * Inserts new view element into currently selected
	 * and selects it
	*/
	public void insertInto (XelfiTreeViewElement element) {
		treeView.insertInto(element);
	}

	/**
	 * Inserts new view element after specified node.
	 * (on the same level in the tree).<br>
	 * Selection remains unchanged.
	*/
	public void insertAfter (XFTreeNode node, XelfiTreeViewElement element) {
		treeView.insertAfter(node, element);
	}

	/**
	 * Inserts new view element into specified node
	 * (item will be first in a childlist).<br>
	 * Selection remains unchanged.
	*/
	public void insertInto (XFTreeNode node, XelfiTreeViewElement element) {
		treeView.insertAfter(node, element);
	}

	/**
	 * Removes currently selected element,
	 * selects next one or previous one
	*/
	public void remove () {
		treeView.remove();
	}

	/**
	 * Pulls currently selected element down or up, if it's possible.
	 * (it means if element can have children)
	*/
	public void pull (boolean state) {
		// inefficient - i'll have to change it
		treeView.pull(state);
	}

	/**
	 * Toggles the pull of currently selected element.
	 * Does nothing when multiple elements are selected.
	*/
	public void togglePull () {
		treeView.togglePull();
	}

	/**
	 * Selects next element in tree (from top to down)
	*/
	public void next () {
		treeView.next();
	}

	/**
	 * Selects previous element in tree (from top to down)
	 * if it is possible.
	*/
	public void previous () {
		treeView.previous();
	}

	/**
	 * Selects a node in tree view.
	 * Additional pulling down is done if necessary.
	*/
	public boolean selectAbsNode (XFTreeNode node) {
		return treeView.selectAbsNode(node);
	}

	/**
	 * Returns currently selected element
	*/
	public XFTreeNode getCurNode () {
		return treeView.getCurNode();
	}

	/**
	 * Returns currently selected item
	*/
	public XelfiTreeViewElement getCurItem () {
		return treeView.getCurItem();
	}

	/**
	 * Returns index of currently selected node
	*/
	public int getCurIndex () {
		return treeView.getCurIndex();
	}

	/**
	 * Returns element at index position (can return only elements
	 * which are pulled down at this time, can not return elements
	 * which are hidden because its parent is pulled up)<br>
	 * Returns null on error.
	*/
	public XFTreeNode getNode (int index) {
		return treeView.getNode(index);
	}

	/**
	 * Returns count of elements in tree.<br>
	 * Doesn't count elements which are hidden by pulling.
	*/
	public int getCount () {
		return treeView.getCount();
	}

	/**
	 * Sets selection to node at index.<br>
	 * Returns false if index position doesn't exist
	*/
	public boolean selectNode (int index) {
		return treeView.selectNode(index);
	}

	/**
	 * Sets selection to node(node mustn't be hidden by pulling)<br>
	 * Returns false if node can't be found
	*/
	public boolean selectNode (XFTreeNode node) {
		return treeView.selectNode(node);
	}

	/**
	 * Returns node which contains specified point.<br>
	 * Returns null if there's no node containing the point.
	*/
	public XFTreeNode getNodeFromPoint (int x, int y) {
		return treeView.getNodeFromPoint(x, y);
	}

	/**
	 * Returns bounds of element at specified index.
	 * (indexes counted from 0 and only pulled down are counted)
	*/
	public Rectangle getAbsElementBounds (int index) {
		return treeView.getAbsElementBounds(index);
	}

	/**
	 * Returns bounds of element at specified index.
	 * (indexes counted from 0 and only pulled down are counted)
	 * Bounds are relative to the getView() point.
	*/
	public Rectangle getElementBounds (int index) {
		return treeView.getElementBounds(index);
	}

	/**
	 * Sets closed folder image.<br>
	 * imageFile must be valid full path to the image file.
	 * When image loading fails, returns false
	*/
	public boolean setFolderImage (String imageFile) {
		return treeView.setFolderImage(imageFile);
	}

	/**
	 * Sets closed folder image.<br>
   * When image loading fails, returns false
	*/
	public boolean setFolderImage (Image image) {
		return treeView.setFolderImage(image);
	}

	/**
	 * Sets open folder image.<br>
	 * imageFile must be valid full path to the image file.
	 * When image loading fails, returns false
	*/
	public boolean setOpenFolderImage (String imageFile) {
		return treeView.setOpenFolderImage(imageFile);
	}

	/**
	 * Sets open folder image.<br>
	 * When image loading fails, returns false
	*/
	public boolean setOpenFolderImage (Image image) {
		return treeView.setOpenFolderImage(image);
	}

	/**
	 * Sets open folder image.<br>
	 * imageFile must be valid full path to the image file.
	 * When image loading fails, returns false
	*/
	public boolean setNodeImage (String imageFile) {
		return treeView.setNodeImage(imageFile);
	}

	/**
	 * Sets open folder image.<br>
	 * When image loading fails, returns false
	*/
	public boolean setNodeImage (Image image) {
		return treeView.setNodeImage(image);
	}

	/**
	 * Returns true if this list allows multiple selections.
	*/
	public boolean allowsMultipleSelections () {
		return treeView.allowsMultipleSelections();
	}

	/**
	 * Returns the number of selected items in the list.
	*/
	public int countSelectedItems () {
		return treeView.countSelectedItems();
	}

	/**
	 * Delete an item at index (only pulled down nodes) 
	 * from the list.<br>
	 * Returns false if specified item couldn't be deleted;
	 * true otherwise.
	*/
  public boolean delItem (int index) {
		return treeView.delItem(index);
	}

	/**
	 * Deselects the item at the specified index.
	 * Returns false if specified item hasn't been selected;
	 * true otherwise.
	*/
	public boolean deselect (int index) {
		return treeView.deselect(index);
	}

	/**
	 * Get the selected item on the list or -1 if no item is selected.
	 * If multiple selections is set, returns index of 
	 * first selected item.
	*/
	public int getSelectedIndex () {
		return treeView.getSelectedIndex();
	}

	/**
	 * Returns the selected indexes on the list.
	*/
	public int[] getSelectedIndexes () {
		return treeView.getSelectedIndexes();
	}

	/**
	 * Returns the selected items on the list.
	*/
	public XelfiTreeViewElement[] getSelectedItems () {
		return treeView.getSelectedItems();
	}

	/**
	 * Returns true if the item at the specified index has been selected; 
	 * false otherwise.
	*/
	public boolean isSelected (int index) {
		return treeView.isSelected(index);
	}

	/**
	 * Conditionally selects or unselects item at the specified index.
	*/
	public void select (int index, boolean selectItem) {
		treeView.select(index, selectItem);
	}

	/**
	 * Enables/disables multiple selections for list.
	*/
	public boolean setMultipleSelections (boolean multiple) {
		return treeView.setMultipleSelections(multiple);
	}

	/**
	 * Makes currently selected item visible.
	 * Gets first selected when multiple-select.
	*/
	public void makeCurrentVisible () {
		treeView.makeCurrentVisible();
	}

	/**
	 * Makes visible an item specified by index.
	 * (performs neccesary scrolling up/down when needed)
	*/
	public void makeVisible (int index) {
		treeView.makeVisible(index);
	}

	/**
	 * Refreshes content of treeview and element bounds.
	 * Needed when preferred size of some element inthe tree is changed.
	*/
	public void refresh () {
		treeView.refresh();
	}

	/**
	 * Returns index of node in tree. (only visible items)
	 * Returns -1 when no such node exists.
	*/
	public int getIndex (XFTreeNode node) {
		return treeView.getIndex(node);
	}

	/**
	 * Deselects all selected items.
	*/
	public void deselectAll () {
		treeView.deselectAll();
		repaint();
	}

	/**
	 * Checks if drag-drop feature is enabled
	*/
	public boolean allowsDragDrop () {
		return treeView.allowsDragDrop();
	}

	/**
	 * Enables/disables drag-drop feature
	*/
	public void setDragDrop (boolean dragDrop) {
		treeView.setDragDrop(dragDrop);
	}

	/**
	 * Moves the source node into destination node
	*/
	public void moveItem (XFTreeNode destination,
 											  XFTreeNode source, XFTreeNode after) {
		treeView.moveItem(destination, source, after);
	}

	/**
	 * Notification, Called by tree view or when reshaping
	 * Checks if something has changed in tree view properties.
	 * If so, update scrollbar values.
	*/
	public void updateScrollbars () {
		Dimension treePrefSize = treeView.preferredSize();
		if ((treePrefSize.height<=treeView.size().height) && 
				(treeView.getViewY()==0)) {
			Debug.debug(Debug.INSPECTOR, "hiding...tree size:"+treePrefSize.height+
													" canvas size:"+treeView.size().height);
			if (vHidden==false) {
				vScrollbar.hide();
				vHidden = true;
        layout();
        treeView.requestFocus();
			}
		} else {
			int x = treeView.getViewX();
			int y = treeView.getViewY();
			if ((y!=(vScrollbar.getValue()*LINE_INCREMENT)) ||
					(oldTreeSize.height!=treePrefSize.height) || 
					(oldTreeSize.height-oldSize.height!=
						treePrefSize.height-size().height))
				vScrollbar.setValues(y/LINE_INCREMENT, treeView.size().height/LINE_INCREMENT+1, 0,
															(treePrefSize.height-treeView.size().height)/LINE_INCREMENT);
			Debug.debug(Debug.INSPECTOR, "showing...tree size:"+treePrefSize.height+
													" canvas size:"+treeView.size().height);
			if (vHidden==true) {
				vScrollbar.show();
				vHidden = false;
				layout();
        treeView.requestFocus();
			}
		}
		// copy preferred dimension to oldTreeSize
		oldTreeSize = new Dimension(treePrefSize);
		oldSize = size();
	}

  public Dimension minimumSize () {
    return treeView.minimumSize();
  }

  public Dimension preferredSize () {
    return treeView.preferredSize();
  }

}
