/**
* Xelfi
*/

package xelfi.design.forms;

import java.io.*;
import java.awt.Component;
import xelfi.editor.Source;
import xelfi.editor.XelfiFile;
import xelfi.top.TopLevel;
import xelfi.design.util.XFDesignConstants;
import xelfi.design.util.VersionInfo;
import xelfi.util.Debug;
import xelfi.util.MessageBox;

/**
*	This class provides loading & saving of forms while differentiating different form types
* (Application/ Frame/ Dialog/ Applet).
* @version 	1.13, 06 Apr 1997
* @author 		Ian Formanek
*/

public class XFFormLoader {

	private static String getFormName(String pathName) {
		if (pathName == null) return null;
		int ind = pathName.lastIndexOf(".");
		if (ind == -1)
			return pathName.substring(pathName.lastIndexOf(File.separator)+1);
		else
			return pathName.substring(pathName.lastIndexOf(File.separator)+1, ind);
	}

	/**
	 * This method checks type of form from given file. It does not load the form.
	 * The file name must be complete name with path and extension ".form".
	 * @param fileName	name of file from which to load the form (must be filename with complete path)
	 * @return 			file type constants (FT_???? from XFDesignConstants)
	 */
	public static int checkFormType(String fileName) {
		DataInputStream dis = null;
		String formType = null;
		try {
			dis = new DataInputStream(new FileInputStream(fileName));
			formType = dis.readUTF();
		} catch (IOException e) {
		} finally {
			try {
				dis.close();
			} catch (IOException e) {
				formType = null;
			}
		}
		if (formType == null)
			return XFDesignConstants.FT_NOFORM;
		if ("Application".compareTo(formType) == 0)
			return XFDesignConstants.FT_APPLICATION;
		if ("Frame".compareTo(formType) == 0)
			return XFDesignConstants.FT_FRAME;
		if ("Dialog".compareTo(formType) == 0)
			return XFDesignConstants.FT_DIALOG;
		if ("Applet".compareTo(formType) == 0)
			return XFDesignConstants.FT_APPLET;
		return XFDesignConstants.FT_NOFORM;
	}

	/**
	 * Loads a new form from given file.
	 * The file name must be complete name with path and extension ".form"
	 * This is high-level method.
	 * @param     xf    file from which to load the form (XelfiFile)
	 * @return          new form of type XFFormFrame, XFFormDialog or XFFormApplet 
	 *                  (depends on what is stored in the file) 
	 *                  or null if there was any problem while loading
	 *                  (the message box with error message was already shown)
	 */
	public static XelfiDesignForm loadForm(XelfiFile xf) {
		Source formSource = null;		
		try {
			formSource = new Source(xf);
		} catch (java.io.IOException e) {
		}
		if (formSource == null) return null;
		return (XelfiDesignForm)formSource.getHandler();
	}

	/**
	 * This method loads a new form from given file.
	 * The file name must be complete name with path and extension ".form"
	 * @param fileName	name of file from which to load the form (must be filename with complete path)
	 * @param source		Source for generated code
	 * @return 				new form of type XFFormFrame, XFFormDialog or XFFormApplet 
	 *	            (depends on what is stored in the file) or null if there was any problem
	 */
	public static XelfiDesignForm loadForm(String fileName, Source formSource)
		throws IOException 
	{
		return loadForm(fileName, formSource, getFormName(fileName));
	}

	/**
	 * Loads a new form from given file.
	 * The file name must be complete name with path and extension ".form"
	 * This is low-level method - it is called from within a Source constructor and TemplateManager.
	 * @param fileName	name of file from which to load the form (must be filename with complete path)
	 * @param source	Source for generated code
	 * @param formName	Desired name of the form
	 * @return 		new form of type XFFormFrame, XFFormDialog or XFFormApplet 
	 *	               (depends on what is stored in the file) or null if there was any problem
	 */
	public static XelfiDesignForm loadForm(String fileName, Source formSource, String formName) 
		throws IOException 
	{
		DataInputStream dis;
		XelfiDesignForm form = null;

		// first, we open stream on given form file
		try {
			dis = new DataInputStream(new FileInputStream(fileName));
		} catch (IOException e) {
			new MessageBox ("Error Loading Form", "Cannot open form file").show ();
			throw new IOException();
		}

		// load formType & version infor from form file
		VersionInfo vi = null;
		String formType = null;
		try {
			formType = dis.readUTF();
			vi = new VersionInfo();
			vi.loadFromStream(dis);
		} catch (IOException e) {
			try {
				dis.close();
			} catch (IOException e2) {
			}
			new MessageBox ("Error Loading Form", "Form file is corrupted").show ();
			throw new IOException();
		}


		// check the version number
		if (!vi.equals(XFDesignConstants.DESIGN_VERSION)) {
			new MessageBox ("Error Loading Form - Bad Version", 
			               "Form saved from version: "+vi+
			               "\nCurrent visual technology version: "+XFDesignConstants.DESIGN_VERSION).show ();
			throw new IOException();
		}

		// check form type
		else
		if ("Application".compareTo(formType) == 0) {
			form = new XFFormFrame(formName,true, formSource);
		}
		else
		if ("Frame".compareTo(formType) == 0) {
			form = new XFFormFrame(formName,false, formSource);
		}
		else
		if ("Dialog".compareTo(formType) == 0) {
			form = new XFFormDialog(TopLevel.getMenu(), formName, formSource);
		}
		else
		if ("Applet".compareTo(formType) == 0) {
			form = new XFFormApplet(TopLevel.getMenu(), formName, formSource);
		}
		else {
			new MessageBox ("Error Loading Form", "Form file is corrupted").show ();
			throw new IOException();
		}
			
		// load form from stream
		try {
			form.loadFromStream(dis);
			form.getSource().clearModified();
		} catch (IOException e) {
			new MessageBox ("Error Loading Form", "Form file is corrupted").show ();
			throw new IOException();
		} finally {
			try {
				dis.close();
			} catch (IOException e) {
			}
		}

		Debug.debug(Debug.FORM, "XFFormLoader: showing form...");
		((Component)form).show();  // show the form
		Debug.debug(Debug.FORM, "XFFormLoader: setting form to inspector...");
		TopLevel.getInspector().inspectableChanged(form);
		Debug.debug(Debug.FORM, "XFFormLoader: showing editor for form...");
		TopLevel.showEditor(form.getSource(), false);	
		return form;
	}

	/**
	 * This method saves given form to given filename.
	 * The file name must be complete name with path and extension ".form" and it should be same 
	 * as a name property of the form (in other words the file name is same as form name with ext. ".form")
	 * @param form			form to save
	 * @param fileName	name of file to which to save the form (must be filename with complete path)
	 * @return 				true if saved O.K., false otherwise
	 */
	public static boolean saveForm(XelfiDesignForm form, String fileName) {
		DataOutputStream dos;
		if (form == null) return false;
		try {
			dos = new DataOutputStream(new FileOutputStream(fileName));
		} catch (IOException e) {
			return false;
		}
		try {
			if (form instanceof XFFormFrame)
				if (((XFFormFrame)form).isApp)
					dos.writeUTF("Application");
				else 
					dos.writeUTF("Frame");
			else
			if (form instanceof XFFormDialog)
				dos.writeUTF("Dialog");
			else
			if (form instanceof XFFormApplet)
				dos.writeUTF("Applet");

			else return false;
			
			XFDesignConstants.DESIGN_VERSION.saveToStream(dos);  // save current design version
			form.saveToStream(dos);
		} catch (IOException e) {
			return false;
		} finally {
			try {
				dos.close();
			} catch (IOException e) {
				return false;
			}
		}

		return true;
	}
}
