/**
* Xelfi
*/

package xelfi.design.forms;

import java.util.Hashtable;
import java.awt.Container;
import java.awt.Component;
import xelfi.design.components.XelfiDesignComponent;
import xelfi.design.components.XelfiVisualContainer;
import xelfi.util.Debug;

/**
* This class manages name space (of variable names) for one form.
* It provides methods for allocating default names, checking whether given name exists, ...
* @version 	1.14, 06 Apr 1997 (from 1.10 returns default names with first letter in lower case)
* @author 	Ian Formanek
*/

class XFNameManager {

////////////////////
// Private things //
////////////////////
	private Hashtable defaultTable = new Hashtable();
	private Hashtable namesTable = new Hashtable();
	
	public XFNameManager() {
		namesTable.put("insets", "insets"); // this name is allocated by default
	}
		
	private String lowerFirst(String s) {
		if (s == null) return null;
		if ("".equals(s)) return "";
		return s.substring(0,1).toLowerCase() + s.substring(1);
	}
	
	/**
	* If the compName = className + int number then returns this number
	* otherwise return -1.
	* Note: Expects that className is already changed to start with lowercase.
	*/
	int getCompNum(String className, String compName) {
		if (compName.startsWith(className)) {
			int num;
			try {
				num = Integer.parseInt(compName.substring(className.length()));
			} catch (NumberFormatException e) { 
				return -1;
			}
			if (num>0) return num;
		}
		return -1;
	}
	
	private boolean checkContainedNames(XelfiVisualContainer cont) {
		Component comps[] = ((Container)cont).getComponents();
		for (int i=0; i<comps.length; i++) {
			if (namesTable.containsKey(((XelfiDesignComponent)comps[i]).getName())) return false;
			if (comps[i] instanceof XelfiVisualContainer) 
				if (!checkContainedNames((XelfiVisualContainer)comps[i]))
					return false;
		}
		return true;
	}
		
///////////////////
// Public things //
///////////////////

	/**
	* Returns whether given name is already used
	*/
	boolean nameExists(String name) {
		return namesTable.containsKey(name);
	}
	
	/**
	* Returns new default name for given component's type
	*/
	String getNewName(XelfiDesignComponent comp) {
		String newName = lowerFirst(comp.getClassName());
		Object maxNum = defaultTable.get(newName);
		if (maxNum == null) {
			defaultTable.put(newName, new Integer(1));
			newName = newName + 1;
			namesTable.put(newName, newName);
			return newName;
		}
		else {
			int newMax = ((Integer)maxNum).intValue()+1;
			defaultTable.put(newName,new Integer(newMax));
			newName = newName + newMax;
			namesTable.put(newName, newName);
			return newName;
		}
	}
	
	/**
	* Inserts given component's name into all names table.
	* Also if it has a default name, default max number for given component's type is updated.
	* @returns false if the name already existed, true if O.K.
	*/
	boolean allocName(XelfiDesignComponent comp) {
		String compName = comp.getName();
		if (namesTable.containsKey(compName)) return false;

		namesTable.put(compName, compName);
		String className = lowerFirst(comp.getClassName());
		int compNum = getCompNum(className, compName);

		// if given component has a default name (i.e. the component's name = class name + int number)
		if (compNum >0) {
			Object maxNum = defaultTable.get(className);
			if ((maxNum == null) || (((Integer)maxNum).intValue()<compNum))
				defaultTable.put(className, new Integer(compNum));
		}
		return true;
		// [PENDING-how should it be used???-is it necessary to check whether this number alreeady exists?]
	}
	
	/**
	* Releases given name (the name becomes free for future use)
	*/
	void releaseName(XelfiDesignComponent comp) {
		String compName = comp.getName();
		if (!namesTable.containsKey(compName)) return;
		
		namesTable.remove(compName);
		String className = lowerFirst(comp.getClassName());
		int compNum = getCompNum(className, compName);

		// if given component has a default name (i.e. the component's name = class name + int number)
		if (compNum >0) {
			Object maxNum = defaultTable.get(className);
			if ((maxNum == null) || (((Integer)maxNum).intValue() == compNum))
				defaultTable.put(className, new Integer(compNum-1));
		}
	}
}