/**
* Xelfi programming environment
*/

package xelfi.editor;
import xelfi.util.BinaryTree;
import xelfi.util.BinaryTreeCompare;
import xelfi.util.DuplicateException;
import java.util.NoSuchElementException;
import java.util.Enumeration;
/**
* Block container is used to maintain list of <EM>nonoverlaping</EM> blocks.
* Its internal structure is optimized for finding block that contains a position.
* When an invalid block (that overlaps the previous one or is in different document)
* is inserted an exception is thrown.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class BlockContainer extends Object implements BinaryTreeCompare {
  /** the binary tree */
  private BinaryTree tree;
  /** the type of containing */
  private byte incl;

  /** Creates new empty block container.
  */
  public BlockContainer () {
    // this class implements the compare method
    tree = new BinaryTree (this);
  };

  /** Compares the blocks.
  * @param b1 the first one
  * @param b2 the second one
  * @return the integer that indentifies the result
  */
  public byte compare (Object b1, Object b2) {
    if (b2 instanceof Position) {
      switch (((Block)b1).compare ((Position)b2)) {
      case Block.DIFFERENT:
      case Block.BEFORE:
        return BinaryTreeCompare.LESS;
      case Block.BEGIN:
        if ((incl & Block.BEGIN) != 0) return BinaryTreeCompare.EQUAL;
        else return BinaryTreeCompare.LESS;
      case Block.INSIDE:
        return BinaryTreeCompare.EQUAL;
      case Block.END:
        if ((incl & Block.END) != 0) return BinaryTreeCompare.EQUAL;
        else return BinaryTreeCompare.GREATER;
      case Block.AFTER:
        return BinaryTreeCompare.GREATER;
      };
    } else {
      Block bl1 = (Block)b1;
      Block bl2 = (Block)b2;

      if (bl1 == bl2) return BinaryTreeCompare.EQUAL;
  
      byte c1 = bl1.compare (bl2.getBegin ());
      switch (c1) {
      case Block.DIFFERENT:
      case Block.AFTER:
      case Block.END:
        return BinaryTreeCompare.GREATER;
      case Block.INSIDE:
      case Block.BEGIN:
        // overlap
        return BinaryTreeCompare.EQUAL;
      case Block.BEFORE:
        if (bl2.getEnd ().lessOrEqual (bl1.getBegin ())) {
          return BinaryTreeCompare.LESS;
        } else {
          // overlap
          return BinaryTreeCompare.EQUAL;
        }
      }
    }
    return BinaryTreeCompare.EQUAL;
  };


  /** Adds new block to the container.
  * @param b the block
  * @exception DuplicateException if the block overlaps with an already inserted or
  * the block is in another document than previously inserted one.
  */
  public void add (Block b) throws DuplicateException {
    tree.insert (b);
  };

  /** Removes a block from container.
  * @param b the block to delete
  * @exception NoSuchElementException if no such block exists
  */
  public void remove (Block o) throws NoSuchElementException {
    tree.delete (o);
  };

  /** Finds a block that contains specified position.
  * @param p the position to find
  * @param incl the type of belonging. It can be composed from constants BEGIN, END,
  * INSIDE by the operator or.
  * @return the block that contains p
  * @exception NoSuchElementException if no such block exists
  * @see Block#BEGIN
  * @see Block#INSIDE
  * @see Block#END
  */
  public synchronized Block find (Position p, int incl) throws NoSuchElementException {
    if (p == null) {
      throw new NoSuchElementException ();
    } else {
      this.incl = (byte) (incl & (Block.BEGIN | Block.INSIDE | Block.END));
      return (Block)tree.access (p);
    }
  }

  /** Finds a block that overlaps with block b.
  * @param b the block to test
  * @return the block that overlaps with b
  * @exception NoSuchElementException if no such block exists. This means
  *  that block b can be inserted into the BlockContainer.
  */
  public synchronized Block find (Block b) throws NoSuchElementException {
    if (b == null) {
      throw new NoSuchElementException ();
    } else {
      return b.isEmpty () ? (Block)tree.access (b.getBegin ()) : (Block)tree.access (b);
    }
  }

  /** All elements in container.
  * @return the enumeration of elements
  */
  public Enumeration elements () {
    return tree.elements ();
  };
};




